@@ -22,15 +22,19 @@
 
 #include <libgda/gda-config.h>
 #include <libgnomedb/gnome-db-data-source-selector.h>
+#include <libgnomedb/gnome-db-dsn-config-druid.h>
+#include <libgnomedb/gnome-db-i18n.h>
 #include <libgnomedb/gnome-db-login.h>
 #include <libgnomedb/gnome-db-util.h>
-#include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
-#include "gnome-db-i18n.h"
+#include <gtk/gtktable.h>
+#include <gtk/gtkstock.h>
 
 struct _GnomeDbLoginPrivate {
 	gchar *dsn_name;
 	GtkWidget *dsn_entry;
+	GtkWidget *create_dsn_button;
+	gboolean hiding_button;
 	GtkWidget *username_entry;
 	GtkWidget *password_entry;
 };
@@ -93,6 +97,7 @@ gnome_db_login_init (GnomeDbLogin *login, GnomeDbLoginClass *klass)
 	/* allocate the internal structure */
 	login->priv = g_new0 (GnomeDbLoginPrivate, 1);
 	login->priv->dsn_name = NULL;
+	login->priv->hiding_button = TRUE;
 }
 
 static void
@@ -179,6 +184,55 @@ gnome_db_login_get_type (void)
 	return type;
 }
 
+static void
+druid_finished_cb (GnomeDbDsnConfigDruid *druid, gboolean error, GnomeDbLogin *login)
+{
+	const GdaDataSourceInfo *dsn_info;
+
+	if (!error) {
+		dsn_info = gnome_db_dsn_config_druid_get_dsn (druid);
+		if (dsn_info) {
+			gnome_db_config_save_data_source (dsn_info->name,
+							  dsn_info->provider,
+							  dsn_info->cnc_string,
+							  dsn_info->description,
+							  dsn_info->username,
+							  dsn_info->password);
+		}
+	}
+
+	gtk_widget_destroy (gtk_widget_get_toplevel (GTK_WIDGET (druid)));
+	gtk_widget_set_sensitive (gtk_widget_get_toplevel (GTK_WIDGET (login)), TRUE);
+}
+
+static void
+add_dsn_cb (GtkButton *button, GnomeDbLogin *login)
+{
+	GtkWidget *druid, *dialog;
+	GdkPixbuf *icon;
+
+	gtk_widget_set_sensitive (gtk_widget_get_toplevel (GTK_WIDGET (login)), FALSE);
+
+	/* create the dialog */
+	dialog = gtk_dialog_new_with_buttons (_("New data source"),
+					      GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (login))),
+					      0, NULL);
+	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
+	if (icon) {
+		gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+		g_object_unref (icon);
+	}
+
+	druid = gnome_db_dsn_config_druid_new ();
+	g_signal_connect (G_OBJECT (druid), "finished",
+			  G_CALLBACK (druid_finished_cb), login);
+	gtk_widget_show (druid);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), druid, TRUE, TRUE, 6);
+
+	/* run the dialog */
+	gtk_widget_show (dialog);
+}
+
 static GtkWidget *
 gnome_db_login_construct (GnomeDbLogin *login, const gchar *dsn)
 {
@@ -192,13 +246,12 @@ gnome_db_login_construct (GnomeDbLogin *login, const gchar *dsn)
 	else
 		login->priv->dsn_name = NULL;
 
-	table = gnome_db_new_table_widget (3, 2, FALSE);
-        gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-        gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	table = gnome_db_new_table_widget (3, 3, FALSE);
 	gtk_box_pack_start (GTK_BOX (login), table, TRUE, TRUE, 0);
 
 	label = gnome_db_new_label_widget (_("Data Source:"));
         gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
         gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	login->priv->dsn_entry = gnome_db_data_source_selector_new (dsn);
@@ -206,21 +259,30 @@ gnome_db_login_construct (GnomeDbLogin *login, const gchar *dsn)
 	gtk_table_attach (GTK_TABLE (table), login->priv->dsn_entry, 1, 2, 0, 1,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
 
+	login->priv->create_dsn_button = gnome_db_new_button_widget_from_stock (GTK_STOCK_ADD);
+	g_signal_connect (G_OBJECT (login->priv->create_dsn_button), "clicked",
+			  G_CALLBACK (add_dsn_cb), login);
+	gtk_widget_hide (login->priv->create_dsn_button);
+	gtk_table_attach (GTK_TABLE (table), login->priv->create_dsn_button, 2, 3, 0, 1,
+			  GTK_FILL, GTK_FILL, 0, 0);
+
 	label = gnome_db_new_label_widget (_("Username:"));
         gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
         gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
 	login->priv->username_entry = gnome_db_new_entry_widget (0, TRUE);
-	gtk_table_attach (GTK_TABLE (table), login->priv->username_entry, 1, 2, 1, 2,
+	gtk_table_attach (GTK_TABLE (table), login->priv->username_entry, 1, 3, 1, 2,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
 
 	label = gnome_db_new_label_widget (_("Password:"));
         gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
         gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
 	login->priv->password_entry = gnome_db_new_entry_widget (0, TRUE);
 	gtk_entry_set_visibility (GTK_ENTRY (login->priv->password_entry), FALSE);
-	gtk_table_attach (GTK_TABLE (table), login->priv->password_entry, 1, 2, 2, 3,
+	gtk_table_attach (GTK_TABLE (table), login->priv->password_entry, 1, 3, 2, 3,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
 
 	return GTK_WIDGET (login);
@@ -252,8 +314,27 @@ gnome_db_login_get_dsn (GnomeDbLogin *login)
 	return gnome_db_option_menu_get_selection (GTK_OPTION_MENU (login->priv->dsn_entry));
 }
 
+/**
+ * gnome_db_login_set_dsn
+ * @login: A #GnomeDbLogin widget.
+ * @dsn: Data source name.
+ *
+ * Set the data source name on the given #GnomeDbLogin widget.
+ */
+void
+gnome_db_login_set_dsn (GnomeDbLogin *login, const gchar *dsn)
+{
+	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
+	gnome_db_option_menu_set_selection (GTK_OPTION_MENU (login->priv->dsn_entry), dsn);
+}
+
 /**
  * gnome_db_login_get_username
+ * @login: A #GnomeDbLogin widget.
+ *
+ * Get the value currently entered in the username text entry.
+ *
+ * Returns: the username being entered in the login widget.
  */
 const gchar *
 gnome_db_login_get_username (GnomeDbLogin *login)
@@ -262,6 +343,20 @@ gnome_db_login_get_username (GnomeDbLogin *login)
 	return gtk_entry_get_text (GTK_ENTRY (login->priv->username_entry));
 }
 
+/**
+ * gnome_db_login_set_username
+ * @login: a #GnomeDbLogin widget.
+ * @username: username to display in the widget.
+ *
+ * Sets the user name to be displayed in the given #GnomeDbLogin widget.
+ */
+void
+gnome_db_login_set_username (GnomeDbLogin *login, const gchar *username)
+{
+	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
+	gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), username);
+}
+
 /**
  * gnome_db_login_get_password
  */
@@ -271,3 +366,40 @@ gnome_db_login_get_password (GnomeDbLogin *login)
 	g_return_val_if_fail (GNOME_DB_IS_LOGIN (login), NULL);
 	return gtk_entry_get_text (GTK_ENTRY (login->priv->password_entry));
 }
+
+/**
+ * gnome_db_login_set_password
+ * @login: a #GnomeDbLogin widget.
+ * @password: password to display in the widget.
+ *
+ * Sets the password to be displayed in the given #GnomeDbLogin widget.
+ */
+void
+gnome_db_login_set_password (GnomeDbLogin *login, const gchar *password)
+{
+	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
+	gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), password);
+}
+
+/**
+ * gnome_db_login_set_enable_create_button
+ * @login: A #GnomeDbLogin widget.
+ * @enable: Flag to enable/disable the button.
+ *
+ * Enable/disable the button for allowing the user to create new data sources
+ * from the login widget. This button is disabled by default, and when enabled,
+ * allows the user to start the database properties configuration applet.
+ */
+void
+gnome_db_login_set_enable_create_button (GnomeDbLogin *login, gboolean enable)
+{
+	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
+
+	if (login->priv->hiding_button && enable) {
+		login->priv->hiding_button = FALSE;
+		gtk_widget_show (login->priv->create_dsn_button);
+	} else if (!login->priv->hiding_button && !enable) {
+		login->priv->hiding_button = TRUE;
+		gtk_widget_hide (login->priv->create_dsn_button);
+	}
+}
